@@ -2,8 +2,7 @@ use super::*;
 use crate::rules::parser::{from_str2, Span};
 
 use indexmap::map::IndexMap;
-
-
+use crate::rules::values::Value::Bool;
 
 
 #[test]
@@ -216,7 +215,7 @@ fn test_base_rule() {
     assert_eq!(
         base_rule(from_str2(example)),
         Ok((span, BaseRule{
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             property_comparison: prop_comparison,
             custom_message: None
         }))
@@ -243,7 +242,7 @@ fn test_conditional_rule() {
     assert_eq!(
         conditional_rule(from_str2(example)),
         Ok((span, ConditionalRule{
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             when_condition,
             check_condition
         }))
@@ -269,14 +268,14 @@ fn test_clause_with_message() {
     };
     let conditional_rule = Rule::Conditional(
         ConditionalRule {
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             when_condition,
             check_condition
         }
     );
     let basic_rule_1 = Rule::Basic(
         BaseRule{
-            type_name: String::from("AWS::EC2::Instance"),
+            type_name: TypeName{type_name: String::from("AWS::EC2::Instance")},
             property_comparison: PropertyComparison {
                 property_path: String::from("InstanceType"),
                 operator: CmpOperator::Eq,
@@ -287,7 +286,7 @@ fn test_clause_with_message() {
     );
     let basic_rule_2 = Rule::Basic(
         BaseRule {
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             property_comparison: PropertyComparison {
                 property_path: String::from("BucketName"),
                 operator: CmpOperator::Eq,
@@ -302,8 +301,9 @@ fn test_clause_with_message() {
         rules: vec![conditional_rule, basic_rule_1, basic_rule_2]
     };
 
+    let parsed_rules = rule_line(from_str2(example));
     assert_eq!(
-        rule_line(from_str2(example)),
+        parsed_rules,
         Ok((span,RuleLineType::Clause(clause)))
     );
 }
@@ -332,14 +332,14 @@ fn test_parse_rules_file() {
     };
     let conditional_rule = Rule::Conditional(
         ConditionalRule {
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             when_condition,
             check_condition
         }
     );
     let basic_rule_1 = Rule::Basic(
         BaseRule{
-            type_name: String::from("AWS::EC2::Instance"),
+            type_name: TypeName{type_name: String::from("AWS::EC2::Instance")},
             property_comparison: PropertyComparison {
                 property_path: String::from("InstanceType"),
                 operator: CmpOperator::Eq,
@@ -351,7 +351,7 @@ fn test_parse_rules_file() {
     let basic_rule_clone = basic_rule_1.clone();
     let basic_rule_2 = Rule::Basic(
         BaseRule {
-            type_name: String::from("AWS::S3::Bucket"),
+            type_name: TypeName{type_name: String::from("AWS::S3::Bucket")},
             property_comparison: PropertyComparison {
                 property_path: String::from("BucketName"),
                 operator: CmpOperator::Eq,
@@ -392,6 +392,122 @@ fn test_parse_rules_file_rule_error() {
     );
 }
 
+#[test]
+fn test_disjunction_basic_clauses() {
+    let example = "let encryption_flag = true \n AWS::EC2::Volume Encrypted == %encryption_flag \n AWS::EC2::Volume Size == 100 |OR| AWS::EC2::Volume Size == 50";
+    let actual_rules = parse_rules_file(&String::from(example), &String::from("file_name")).unwrap();
+
+    let encryption_rule = Rule::Basic(
+        BaseRule{
+            type_name: TypeName{type_name: String::from("AWS::EC2::Volume")},
+            property_comparison: PropertyComparison {
+                property_path: String::from("Encrypted"),
+                operator: CmpOperator::Eq,
+                comparison_value: OldGuardValues::VariableAccess(String::from("encryption_flag"))
+            },
+            custom_message: None
+        }
+    );
+
+    let volume_size_100 = Rule::Basic(
+        BaseRule{
+            type_name: TypeName{type_name: String::from("AWS::EC2::Volume")},
+            property_comparison: PropertyComparison {
+                property_path: String::from("Size"),
+                operator: CmpOperator::Eq,
+                comparison_value: OldGuardValues::Value(Value::Int(100))
+            },
+            custom_message: None
+        }
+    );
+
+    let volume_size_50 = Rule::Basic(
+        BaseRule{
+            type_name: TypeName{type_name: String::from("AWS::EC2::Volume")},
+            property_comparison: PropertyComparison {
+                property_path: String::from("Size"),
+                operator: CmpOperator::Eq,
+                comparison_value: OldGuardValues::Value(Value::Int(50))
+            },
+            custom_message: None
+        }
+    );
+
+    let expected_rules = vec![
+        RuleLineType::Assignment(Assignment {
+            var_name: String::from("encryption_flag"),
+            value: OldGuardValues::Value(Value::Bool(true))
+        }),
+        RuleLineType::Clause(Clause {
+            rules: vec![encryption_rule]
+        }),
+        RuleLineType::Clause(Clause {
+            rules: vec![volume_size_100, volume_size_50]
+        })
+    ];
+
+    assert_eq!(
+        actual_rules,
+        expected_rules
+    );
+}
+
+#[test]
+fn test_disjunction_conditional_clauses() {
+    let example = r#"AWS::EC2::Instance WHEN InstanceType == "m2.large" CHECK .DeletionPolicy == Retain |OR| AWS::EC2::Instance WHEN InstanceType == "t2.micro" CHECK .Encrypted == true"#;
+    let actual_rules = parse_rules_file(&String::from(example), &String::from("file_name")).unwrap();
+
+    let m2_when_condition = PropertyComparison{
+        property_path: String::from("InstanceType"),
+        operator: CmpOperator::Eq,
+        comparison_value: OldGuardValues::Value(Value::String(String::from("m2.large")))
+    };
+
+    let m2_check_condition = PropertyComparison{
+        property_path: String::from(".DeletionPolicy"),
+        operator: CmpOperator::Eq,
+        comparison_value: OldGuardValues::Value(Value::String(String::from("Retain")))
+    };
+
+    let m2_conditional_rule = Rule::Conditional(
+        ConditionalRule {
+            type_name: TypeName{type_name: String::from("AWS::EC2::Instance")},
+            when_condition: m2_when_condition,
+            check_condition: m2_check_condition
+        }
+    );
+
+    let t2_when_condition = PropertyComparison{
+        property_path: String::from("InstanceType"),
+        operator: CmpOperator::Eq,
+        comparison_value: OldGuardValues::Value(Value::String(String::from("t2.micro")))
+    };
+
+    let t2_check_condition = PropertyComparison{
+        property_path: String::from(".Encrypted"),
+        operator: CmpOperator::Eq,
+        comparison_value: OldGuardValues::Value(Value::Bool(true))
+    };
+
+    let t2_conditional_rule = Rule::Conditional(
+        ConditionalRule {
+            type_name: TypeName{type_name: String::from("AWS::EC2::Instance")},
+            when_condition: t2_when_condition,
+            check_condition: t2_check_condition
+        }
+    );
+
+    let expected_rules = vec![
+        RuleLineType::Clause(Clause {
+            rules: vec![m2_conditional_rule, t2_conditional_rule]
+        })
+    ];
+
+    assert_eq!(
+        actual_rules,
+        expected_rules
+    );
+}
 
 fn make_empty_span(offset: usize) -> Span<'static> {
     unsafe { Span::new_from_raw_offset(offset, 1, "", "") }
