@@ -54,7 +54,7 @@ impl EvaluationContext for DummyEval {
         unimplemented!()
     }
 
-    fn end_evaluation(&self, _eval_type: EvaluationType, _context: &str, _msg: String, _from: Option<PathAwareValue>, _to: Option<PathAwareValue>, _status: Option<Status>) {
+    fn end_evaluation(&self, _eval_type: EvaluationType, _context: &str, _msg: String, _from: Option<PathAwareValue>, _to: Option<PathAwareValue>, _status: Option<Status>, _cmp: Option<(CmpOperator, bool)>) {
     }
 
     fn start_evaluation(&self, _eval_type: EvaluationType, _context: &str) {
@@ -147,10 +147,10 @@ impl<'r> EvaluationContext for Reporter<'r> {
         self.0.rule_status(rule_name)
     }
 
-    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>) {
+    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>, cmp: Option<(CmpOperator, bool)>) {
         println!("{} {} {:?}", eval_type, context, status);
         self.0.end_evaluation(
-            eval_type, context, msg, from, to, status
+            eval_type, context, msg, from, to, status, cmp
         )
     }
 
@@ -897,8 +897,8 @@ impl<'a, 'b> EvaluationContext for VariableResolver<'a, 'b> {
         self.0.rule_status(rule_name)
     }
 
-    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>) {
-        self.0.end_evaluation(eval_type, context, msg, from, to, status);
+    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>, cmp: Option<(CmpOperator, bool)>) {
+        self.0.end_evaluation(eval_type, context, msg, from, to, status, cmp);
     }
 
     fn start_evaluation(&self, eval_type: EvaluationType, context: &str) {
@@ -1372,15 +1372,15 @@ fn test_compare_loop_atleast_one_eq() -> Result<()> {
     //
     // match any one rhs = false, at-least-one = false
     //
-    let (result, _first, _with) = compare_loop(
+    let (result, _results) = compare_loop(
         &lhs_values, &rhs_values, path_value::compare_eq, false, false
     )?;
     assert_eq!(result, false);
 
     //
-    // match any one rhs = false, at-least-one = false
+    // match any one rhs = false, at-least-one = true
     //
-    let (result, _first, _with) = compare_loop(
+    let (result, _results) = compare_loop(
         &lhs_values, &rhs_values, path_value::compare_eq, false, true
     )?;
     assert_eq!(result, true);
@@ -1388,7 +1388,7 @@ fn test_compare_loop_atleast_one_eq() -> Result<()> {
     //
     // match any one rhs = true, at-least-one = false
     //
-    let (result, _first, _with) = compare_loop(
+    let (result, _results) = compare_loop(
         &lhs_values, &rhs_values, path_value::compare_eq, true, false
     )?;
     assert_eq!(result, false);
@@ -1396,6 +1396,169 @@ fn test_compare_loop_atleast_one_eq() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn test_compare_loop_all() -> Result<()> {
+    let root = Path::root();
+    let lhs = [
+        PathAwareValue::String((root.clone(), "aws:isSecure".to_string())),
+        PathAwareValue::String((root.clone(), "aws:sourceVpc".to_string())),
+    ];
+    let rhs = [
+        PathAwareValue::Regex((root.clone(), "aws:[sS]ource(Vpc|VPC|Vpce|VPCE)".to_string())),
+    ];
+
+    let lhs_values = lhs.iter().collect::<Vec<&PathAwareValue>>();
+    let rhs_values = rhs.iter().collect::<Vec<&PathAwareValue>>();
+
+    let results = super::compare_loop_all(&lhs_values, &rhs_values, path_value::compare_eq, false)?;
+    //
+    // One result for each LHS value
+    //
+    assert_eq!(results.1.len(), 2);
+    let (outcome, from, to) = &results.1[0];
+    assert_eq!(*outcome, false);
+    assert_eq!(from, &Some(lhs[0].clone()));
+    assert_eq!(to, &Some(rhs[0].clone()));
+
+    let (outcome, from, to) = &results.1[1];
+    assert_eq!(*outcome, true);
+    assert_eq!(from, &None);
+    assert_eq!(to, &None);
+
+    Ok(())
+}
+
+#[test]
+fn test_compare_lists() -> Result<()> {
+    let root = Path::root();
+    let value = PathAwareValue::List((
+        root.clone(),
+        vec![
+            PathAwareValue::Int((root.clone(), 1)),
+            PathAwareValue::Int((root.clone(), 2))
+        ]));
+    let lhs = vec![&value];
+    let rhs = vec![&value];
+
+    let query = [];
+    let r = super::compare(
+        &lhs,
+        &query,
+        &rhs,
+        None,
+        super::super::path_value::compare_eq,
+        false,
+        false
+    )?;
+    assert_eq!(r.0, Status::PASS);
+    Ok(())
+}
+
+#[test]
+fn test_compare_rulegen() -> Result<()> {
+    let rulegen_created = r###"
+let aws_ec2_securitygroup_resources = Resources.*[ Type == 'AWS::EC2::SecurityGroup' ]
+rule aws_ec2_securitygroup when %aws_ec2_securitygroup_resources !empty {
+  %aws_ec2_securitygroup_resources.Properties.SecurityGroupEgress == [{"CidrIp":"0.0.0.0/0","IpProtocol":-1},{"CidrIpv6":"::/0","IpProtocol":-1}]
+}"###;
+    let template = r###"
+Resources:
+
+  # SecurityGroups
+  ## Alb Security Groups
+
+  rFrontendAppSpecificSg:
+    Type: AWS::EC2::SecurityGroup
+    Properties:
+      GroupDescription: Frontend Security Group
+      GroupName: secgrp-frontend
+      SecurityGroupEgress:
+        - CidrIp: "0.0.0.0/0"
+          IpProtocol: -1
+        - CidrIpv6: "::/0"
+          IpProtocol: -1
+      VpcId: vpc-123abc
+    "###;
+    let rules = RulesFile::try_from(rulegen_created)?;
+    let value = PathAwareValue::try_from(serde_yaml::from_str::<serde_json::Value>(template)?)?;
+    let root = RootScope::new(&rules, &value);
+    let status = rules.evaluate(&value, &root)?;
+    assert_eq!(status, Status::PASS);
+    Ok(())
+}
+
+#[test]
+fn test_guard_10_compatibility_and_diff() -> Result<()> {
+    let value_str = r###"
+    Statement:
+      - Principal: ['*', 's3:*']
+    "###;
+    let value = PathAwareValue::try_from(
+        serde_yaml::from_str::<serde_json::Value>(value_str)?)?;
+    let dummy = DummyEval{};
+
+    //
+    // Evaluation differences with 1.0 for Statement.*.Principal == '*'
+    //
+    // Guard 1.0 this would PASS with at-least one semantics for the payload above. This is where docs
+    // need to be consulted to understand that == is at-least-one and != is ALL. Due to this decision certain
+    // expressions like ensure that ALL AWS::EC2::Volume Encrypted == true, could not be specified
+    //
+    // In Guard 2.0 this would FAIL. The reason being that Guard 2.0 goes for explicitness in specifying
+    // clauses. By default it asserts for ALL semantics. If you expecting to match at-least one or more
+    // you must use SOME keyword that would evaluate correctly. With this support in 2.0 we can
+    // support ALL expressions like
+    //
+    //        AWS::EC2::Volume Properties.Encrypted == true
+    //
+    // At the same time, one can explicitly express at-least-one or more semantics using SOME
+    //
+    //         AWS::EC2::Volume SOME Properties.Encrypted == true
+    //
+    // And finally
+    //
+    //       AWS::EC2::Volume Properties {
+    //             Encrypted !EXISTS or
+    //             Encrypted == true
+    //       }
+    //
+    // can be correctly specified. This also makes the intent clear to both the rule author and
+    // auditor what was acceptable. Here, it is okay that accept Encrypted was not specified
+    // as an attribute or when specified it must be true. This makes it clear to the reader/auditor
+    // rather than guess at how Guard engine evaluates.
+    //
+    // The evaluation engine is purposefully dumb and stupid, defaults to working
+    // one way consistently enforcing ALL semantics. Needs to told explicitly to do otherwise
+    //
+
+    let clause_str = r#"Statement.*.Principal == '*'"#;
+    let clause = GuardClause::try_from(clause_str)?;
+    let status = clause.evaluate(&value, &dummy)?;
+    assert_eq!(status, Status::FAIL);
+
+    let clause_str = r#"SOME Statement.*.Principal == '*'"#;
+    let clause = GuardClause::try_from(clause_str)?;
+    let dummy = DummyEval{};
+    let status = clause.evaluate(&value, &dummy)?;
+    assert_eq!(status, Status::PASS);
+
+    let value_str = r###"
+    Statement:
+      - Principal: aws
+      - Principal: ['*', 's3:*']
+    "###;
+    let value = PathAwareValue::try_from(
+        serde_yaml::from_str::<serde_json::Value>(value_str)?)?;
+    //
+    // Evaluate the SOME clause again, it must pass with ths value as well
+    //
+    let status = clause.evaluate(&value, &dummy)?;
+    assert_eq!(status, Status::PASS);
+
+
+    Ok(())
+}
+
 #[test]
 fn block_evaluation() -> Result<()> {
     let value_str = r#"
@@ -1636,8 +1799,8 @@ impl<'a> EvaluationContext for Tracker<'a> {
         self.root.rule_status(rule_name)
     }
 
-    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>) {
-        self.root.end_evaluation(eval_type, context, msg, from, to, status.clone());
+    fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>, cmp: Option<(CmpOperator, bool)>) {
+        self.root.end_evaluation(eval_type, context, msg, from, to, status.clone(), cmp);
         if eval_type == EvaluationType::Rule {
             match self.expected.get(context) {
                 Some(e) => {
@@ -1897,3 +2060,204 @@ fn test_inner_when_skipped() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_multiple_valued_clause_reporting() -> Result<()> {
+    let rule = r###"
+    rule name_check { Resources.*.Properties.Name == /NAME/ }
+    "###;
+
+    let value = r###"
+    Resources:
+      second:
+        Properties:
+          Name: FAILEDMatch
+      first:
+        Properties:
+          Name: MatchNAME
+      matches:
+        Properties:
+          Name: MatchNAME
+      failed:
+        Properties:
+          Name: FAILEDMatch
+    "###;
+
+    #[derive(Debug, Clone)]
+    struct Reporter {};
+    impl EvaluationContext for Reporter {
+        fn resolve_variable(&self, variable: &str) -> Result<Vec<&PathAwareValue>> {
+            todo!()
+        }
+
+        fn rule_status(&self, rule_name: &str) -> Result<Status> {
+            todo!()
+        }
+
+        fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>, _cmp: Option<(CmpOperator, bool)>) {
+            if eval_type == EvaluationType::Clause {
+                match &status {
+                    Some(Status::FAIL) => {
+                        assert_eq!(from.is_some(), true);
+                        assert_eq!(to.is_some(), true);
+                        let path_val = from.unwrap();
+                        let path = path_val.self_path();
+                        assert_eq!(path.0.contains("/second") || path.0.contains("/failed"), true);
+                    },
+                    Some(Status::PASS) => {
+                        assert_eq!(from, None);
+                        assert_eq!(to, None);
+                        assert_eq!(msg.contains("DEFAULT"), true);
+                    }
+                    _ => {}
+                }
+            }
+        }
+
+        fn start_evaluation(&self, eval_type: EvaluationType, context: &str) {
+        }
+    }
+
+    let rules = Rule::try_from(rule)?;
+    let values = PathAwareValue::try_from(serde_yaml::from_str::<serde_json::Value>(value)?)?;
+    let reporter = Reporter{};
+    let status = rules.evaluate(&values, &reporter)?;
+    assert_eq!(status, Status::FAIL);
+    Ok(())
+}
+
+#[test]
+fn test_multiple_valued_clause_reporting_var_access() -> Result<()> {
+    let rule = r###"
+    let resources = Resources.*
+    rule name_check { %resources.Properties.Name == /NAME/ }
+    "###;
+
+    let value = r###"
+    Resources:
+      second:
+        Properties:
+          Name: FAILEDMatch
+      first:
+        Properties:
+          Name: MatchNAME
+      matches:
+        Properties:
+          Name: MatchNAME
+      failed:
+        Properties:
+          Name: FAILEDMatch
+    "###;
+
+    struct Reporter<'a> { root: &'a dyn EvaluationContext };
+    impl<'a> EvaluationContext for Reporter<'a> {
+        fn resolve_variable(&self, variable: &str) -> Result<Vec<&PathAwareValue>> {
+            self.root.resolve_variable(variable)
+        }
+
+        fn rule_status(&self, rule_name: &str) -> Result<Status> {
+            self.root.rule_status(rule_name)
+        }
+
+        fn end_evaluation(&self, eval_type: EvaluationType, context: &str, msg: String, from: Option<PathAwareValue>, to: Option<PathAwareValue>, status: Option<Status>, cmp: Option<(CmpOperator, bool)>) {
+            if eval_type == EvaluationType::Clause {
+                match &status {
+                    Some(Status::FAIL) => {
+                        assert_eq!(from.is_some(), true);
+                        assert_eq!(to.is_some(), true);
+                        let path_val = from.as_ref().unwrap();
+                        let path = path_val.self_path();
+                        assert_eq!(path.0.contains("/second") || path.0.contains("/failed"), true);
+                    },
+                    Some(Status::PASS) => {
+                        assert_eq!(from, None);
+                        assert_eq!(to, None);
+                        assert_eq!(msg.contains("DEFAULT"), true);
+                    }
+                    _ => {}
+                }
+            }
+            self.root.end_evaluation(eval_type, context, msg, from, to, status, cmp)
+        }
+
+        fn start_evaluation(&self, eval_type: EvaluationType, context: &str) {
+            self.root.start_evaluation(eval_type, context)
+        }
+    }
+
+    let rules = RulesFile::try_from(rule)?;
+    let values = PathAwareValue::try_from(serde_yaml::from_str::<serde_json::Value>(value)?)?;
+    let root = RootScope::new(&rules, &values);
+    let reporter = Reporter{ root: &root };
+    let status = rules.evaluate(&values, &reporter)?;
+    assert_eq!(status, Status::FAIL);
+    Ok(())
+}
+
+#[test]
+fn test_in_comparison_operator_for_list_of_lists() -> Result<()> {
+    let template = r###"
+    Resources:
+    MasterRecord:
+        Type: AWS::Route53::RecordSet
+        Properties:
+            HostedZoneName: !Ref 'HostedZoneName'
+            Comment: DNS name for my instance.
+            Name: !Join ['', [!Ref 'SubdomainMaster', ., !Ref 'HostedZoneName']]
+            Type: A
+            TTL: '900'
+            ResourceRecords:
+                - !GetAtt Master.PrivateIp
+    InternalRecord:
+        Type: AWS::Route53::RecordSet
+        Properties:
+            HostedZoneName: !Ref 'HostedZoneName'
+            Comment: DNS name for my instance.
+            Name: !Join ['', [!Ref 'SubdomainInternal', ., !Ref 'HostedZoneName']]
+            Type: A
+            TTL: '900'
+            ResourceRecords:
+                - !GetAtt Master.PrivateIp
+    SubdomainRecord:
+        Type: AWS::Route53::RecordSet
+        Properties:
+            HostedZoneName: !Ref 'HostedZoneName'
+            Comment: DNS name for my instance.
+            Name: !Join ['', [!Ref 'SubdomainDefault', ., !Ref 'HostedZoneName']]
+            Type: A
+            TTL: '900'
+            ResourceRecords:
+                - !GetAtt Infra1.PrivateIp
+    WildcardRecord:
+        Type: AWS::Route53::RecordSet
+        Properties:
+            HostedZoneName: !Ref 'HostedZoneName'
+            Comment: DNS name for my instance.
+            Name: !Join ['', [!Ref 'SubdomainWild', ., !Ref 'HostedZoneName']]
+            Type: A
+            TTL: '900'
+            ResourceRecords:
+                - !GetAtt Infra1.PrivateIp
+    "###;
+
+    let rules = r###"
+    let aws_route53_recordset_resources = Resources.*[ Type == 'AWS::Route53::RecordSet' ]
+    rule aws_route53_recordset when %aws_route53_recordset_resources !empty {
+      %aws_route53_recordset_resources.Properties.Comment == "DNS name for my instance."
+      let targets = [["",["SubdomainWild",".","HostedZoneName"]], ["",["SubdomainInternal",".","HostedZoneName"]], ["",["SubdomainMaster",".","HostedZoneName"]], ["",["SubdomainDefault",".","HostedZoneName"]]]
+      %aws_route53_recordset_resources.Properties.Name IN %targets
+      %aws_route53_recordset_resources.Properties.Type == "A"
+      %aws_route53_recordset_resources.Properties.ResourceRecords IN [["Master.PrivateIp"], ["Infra1.PrivateIp"]]
+      %aws_route53_recordset_resources.Properties.TTL == "900"
+      %aws_route53_recordset_resources.Properties.HostedZoneName == "HostedZoneName"
+    }
+    "###;
+
+    let value = PathAwareValue::try_from(serde_yaml::from_str::<serde_json::Value>(template)?)?;
+    let rule_eval = RulesFile::try_from(rules)?;
+    let context = RootScope::new(&rule_eval, &value);
+    let status = rule_eval.evaluate(&value, &context)?;
+    assert_eq!(status, Status::PASS);
+
+    Ok(())
+}
\ No newline at end of file
